==> human-protocol-sdk-1.0.27.tar.gz: human-protocol-sdk-1.0.27/test/human_protocol_sdk/test_storage.py <==
import hashlib
import json
import random
import unittest
from unittest.mock import MagicMock, patch
import types
from minio import S3Error

from human_protocol_sdk.storage import (
    Credentials,
    StorageClient,
    StorageClientError,
    StorageFileNotFoundError,
)
import requests


class TestCredentials(unittest.TestCase):
    def test_credentials(self):
        credentials = Credentials(
            access_key="my-access-key", secret_key="my-secret-key"
        )
        self.assertEqual(credentials.access_key, "my-access-key")
        self.assertEqual(credentials.secret_key, "my-secret-key")


class TestStorageClient(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.endpoint_url = "s3.us-west-2.amazonaws.com"
        cls.bucket = "my-bucket"
        cls.files = ["file1.txt", "file2.txt"]
        cls.region = "us-west-2"
        cls.credentials = Credentials(
            access_key="my-access-key", secret_key="my-secret-key"
        )

    def setUp(self):
        self.client = StorageClient(
            endpoint_url=self.endpoint_url,
            region=self.region,
            credentials=self.credentials,
        )

    def test_init_authenticated_access(self):
        with patch("human_protocol_sdk.storage.Minio") as mock_client:
            client = StorageClient(
                endpoint_url=self.endpoint_url,
                region=self.region,
                credentials=self.credentials,
            )
            mock_client.assert_called_once_with(
                access_key=self.credentials.access_key,
                secret_key=self.credentials.secret_key,
                region=self.region,
                endpoint=self.endpoint_url,
                secure=True,
            )
            self.assertIsNotNone(client.client)

    def test_init_anonymous_access(self):
        with patch("human_protocol_sdk.storage.Minio") as mock_client:
            client = StorageClient(
                endpoint_url=self.endpoint_url,
            )
            mock_client.assert_called_once()
            self.assertEqual(
                mock_client.call_args_list[0].kwargs["region"], None)
            self.assertEqual(
                mock_client.call_args_list[0].kwargs["endpoint"], self.endpoint_url
            )
            self.assertIsNotNone(client.client)

    def test_init_error(self):
        # Connection error
        with patch("human_protocol_sdk.storage.Minio") as mock_client:
            mock_client.side_effect = Exception("Connection error")
            with self.assertRaises(Exception):
                StorageClient(endpoint_url=self.endpoint_url)

    def test_download_file_from_url(self):
        with patch('requests.get') as mock_get:
            mock_response = mock_get.return_value
            mock_response.raise_for_status.return_value = None
            mock_response.content = b'Test file content'
            url = 'https://www.example.com/file.txt'

            result = StorageClient.download_file_from_url(url)

            self.assertEqual(result, b'Test file content')

    def test_download_file_from_url_invalid_url(self):
        url = 'invalid_url'

        with self.assertRaises(StorageClientError) as cm:
            StorageClient.download_file_from_url(url)
        self.assertEqual(f"Invalid URL: {url}", str(cm.exception))

    def test_download_file_from_url_error(self):
        with patch('requests.get') as mock_get:
            url = 'https://www.example.com/file.txt'
            mock_response = mock_get.return_value
            mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError(
                f"Not Found for url: {url}", response=mock_response)

            with self.assertRaises(StorageClientError) as cm:
                StorageClient.download_file_from_url(url)
            self.assertEqual(f"Not Found for url: {url}", str(cm.exception))

    def test_download_files(self):
        expected_result = [b"file1 contents", b"file2 contents"]
        self.client.client.get_object = MagicMock(
            side_effect=[
                MagicMock(read=MagicMock(return_value=expected_result[0])),
                MagicMock(read=MagicMock(return_value=expected_result[1])),
            ]
        )
        result = self.client.download_files(
            files=self.files, bucket=self.bucket)
        self.assertEqual(result, expected_result)

    def test_download_files_error(self):
        self.client.client.get_object = MagicMock(
            side_effect=S3Error(
                code="NoSuchKey",
                message="Key not found",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageFileNotFoundError):
            self.client.download_files(files=self.files, bucket=self.bucket)

    def test_download_files_anonymous_error(self):
        self.client.client.get_object = MagicMock(
            side_effect=S3Error(
                code="InvalidAccessKeyId",
                message="Access denied",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageClientError):
            self.client.download_files(files=self.files, bucket=self.bucket)

    def test_download_files_exception(self):
        self.client.client.get_object = MagicMock(
            side_effect=Exception("Connection error")
        )
        with self.assertRaises(StorageClientError):
            self.client.download_files(files=self.files, bucket=self.bucket)

    def test_upload_files(self):
        file3 = "file3 content"
        data_sha = hashlib.sha1(json.dumps(
            "file3 content").encode("utf-8")).hexdigest()
        key3 = f"s3{data_sha}.json"

        self.client.client.stat_object = MagicMock(
            side_effect=S3Error(
                code="NoSuchKey",
                message="Object does not exist",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        self.client.client.put_object = MagicMock()
        result = self.client.upload_files(files=[file3], bucket=self.bucket)
        self.assertEqual(result, [key3])

    def test_upload_files_exist(self):
        file3 = "file3 content"
        data_sha = hashlib.sha1(json.dumps(
            "file3 content").encode("utf-8")).hexdigest()
        key3 = f"s3{data_sha}.json"

        self.client.client.stat_object = MagicMock(
            side_effect=[{"_object_name": "1234567890"}]
        )
        self.client.client.put_object = MagicMock()
        result = self.client.upload_files(files=[file3], bucket=self.bucket)
        self.assertEqual(result, [key3])

    def test_upload_files_error(self):
        file3 = "file3 content"

        # HeadObject error
        self.client.client.head_object = MagicMock(
            side_effect=S3Error(
                code="InvalidAccessKeyId",
                message="Access denied",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageClientError):
            self.client.upload_files(files=[file3], bucket=self.bucket)

        # PutObject error
        self.client.client.upload_fileobj = MagicMock(
            side_effect=S3Error(
                code="InvalidAccessKeyId",
                message="Access denied",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageClientError):
            self.client.upload_files(files=[file3], bucket=self.bucket)

    def test_bucket_exists(self):
        self.client.client.bucket_exists = MagicMock(side_effect=[True])
        result = self.client.bucket_exists(bucket=self.bucket)
        self.assertEqual(result, True)

    def test_bucket_exists_anonymous(self):
        client = StorageClient(
            endpoint_url=self.endpoint_url,
        )
        client.client.bucket_exists = MagicMock(side_effect=[True])
        result = client.bucket_exists(bucket=self.bucket)
        self.assertEqual(result, True)

    def test_bucket_not_exists(self):
        self.client.client.bucket_exists = MagicMock(side_effect=[False])
        result = self.client.bucket_exists(bucket=self.bucket)
        self.assertEqual(result, False)

    def test_bucket_error(self):
        self.client.client.bucket_exists = MagicMock(
            side_effect=Exception("Connection error")
        )
        with self.assertRaises(StorageClientError):
            self.client.bucket_exists(bucket=self.bucket)

    def test_list_objects(self):
        file1 = types.SimpleNamespace()
        file2 = types.SimpleNamespace()
        file1._object_name = "file1"
        file2._object_name = "file2"
        self.client.client.list_objects = MagicMock(
            side_effect=[[file1, file2]])
        result = self.client.list_objects(bucket=self.bucket)
        self.assertEqual(result, ["file1", "file2"])

    def test_list_objects_anonymous(self):
        client = StorageClient(
            endpoint_url=self.endpoint_url,
        )
        file1 = types.SimpleNamespace()
        file2 = types.SimpleNamespace()
        file1._object_name = "file1"
        file2._object_name = "file2"
        client.client.list_objects = MagicMock(side_effect=[[file1, file2]])
        result = client.list_objects(bucket=self.bucket)
        self.assertEqual(result, ["file1", "file2"])

    def test_list_objects_empty(self):
        self.client.client.list_objects = MagicMock(side_effect=[[]])
        result = self.client.list_objects(bucket=self.bucket)
        self.assertEqual(result, [])

    def test_list_objects_error(self):
        self.client.client.head_bucket = MagicMock(
            side_effect=Exception("Connection error")
        )
        with self.assertRaises(StorageClientError):
            self.client.list_objects(bucket=self.bucket)

    def test_list_objects_length(self):
        expected_length = random.randint(1, 10)
        mock_client = MagicMock()
        mock_client.list_objects.return_value = [
            types.SimpleNamespace(_object_name=f"file{i}")
            for i in range(expected_length)
        ]
        with patch("human_protocol_sdk.storage.Minio", return_value=mock_client):
            client = StorageClient(
                endpoint_url="https://example.com", credentials=None)
            object_list = client.list_objects(bucket="my-bucket")
            self.assertEqual(len(object_list), expected_length)

    def test_list_objects_length_error(self):
        expected_length = random.randint(1, 10)
        mock_client = MagicMock()
        mock_client.list_objects.return_value = [
            types.SimpleNamespace(_object_name=f"file{i}")
            for i in range(expected_length)
        ]
        with patch("human_protocol_sdk.storage.Minio", return_value=mock_client):
            client = StorageClient(
                endpoint_url="https://example.com", credentials=None)
            object_list = client.list_objects(bucket="my-bucket")
            if len(object_list) != expected_length:
                raise AssertionError(
                    f"Expected {expected_length} objects, but found {len(object_list)}"
                )

==> human-protocol-sdk-1.0.30.tar.gz: human-protocol-sdk-1.0.30/test/human_protocol_sdk/test_storage.py <==
import hashlib
import json
import random
import unittest
from unittest.mock import MagicMock, patch
import types
from minio import S3Error

from human_protocol_sdk.storage import (
    Credentials,
    StorageClient,
    StorageClientError,
    StorageFileNotFoundError,
)
import requests


class TestCredentials(unittest.TestCase):
    def test_credentials(self):
        credentials = Credentials(
            access_key="my-access-key", secret_key="my-secret-key"
        )
        self.assertEqual(credentials.access_key, "my-access-key")
        self.assertEqual(credentials.secret_key, "my-secret-key")


class TestStorageClient(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.endpoint_url = "s3.us-west-2.amazonaws.com"
        cls.bucket = "my-bucket"
        cls.files = ["file1.txt", "file2.txt"]
        cls.region = "us-west-2"
        cls.credentials = Credentials(
            access_key="my-access-key", secret_key="my-secret-key"
        )

    def setUp(self):
        self.client = StorageClient(
            endpoint_url=self.endpoint_url,
            region=self.region,
            credentials=self.credentials,
        )

    def test_init_authenticated_access(self):
        with patch("human_protocol_sdk.storage.Minio") as mock_client:
            client = StorageClient(
                endpoint_url=self.endpoint_url,
                region=self.region,
                credentials=self.credentials,
            )
            mock_client.assert_called_once_with(
                access_key=self.credentials.access_key,
                secret_key=self.credentials.secret_key,
                region=self.region,
                endpoint=self.endpoint_url,
                secure=True,
            )
            self.assertIsNotNone(client.client)

    def test_init_anonymous_access(self):
        with patch("human_protocol_sdk.storage.Minio") as mock_client:
            client = StorageClient(
                endpoint_url=self.endpoint_url,
            )
            mock_client.assert_called_once()
            self.assertEqual(mock_client.call_args_list[0].kwargs["region"], None)
            self.assertEqual(
                mock_client.call_args_list[0].kwargs["endpoint"], self.endpoint_url
            )
            self.assertIsNotNone(client.client)

    def test_init_error(self):
        # Connection error
        with patch("human_protocol_sdk.storage.Minio") as mock_client:
            mock_client.side_effect = Exception("Connection error")
            with self.assertRaises(Exception):
                StorageClient(endpoint_url=self.endpoint_url)

    def test_download_file_from_url(self):
        with patch("requests.get") as mock_get:
            mock_response = mock_get.return_value
            mock_response.raise_for_status.return_value = None
            mock_response.content = b"Test file content"
            url = "https://www.example.com/file.txt"

            result = StorageClient.download_file_from_url(url)

            self.assertEqual(result, b"Test file content")

    def test_download_file_from_url_invalid_url(self):
        url = "invalid_url"

        with self.assertRaises(StorageClientError) as cm:
            StorageClient.download_file_from_url(url)
        self.assertEqual(f"Invalid URL: {url}", str(cm.exception))

    def test_download_file_from_url_error(self):
        with patch("requests.get") as mock_get:
            url = "https://www.example.com/file.txt"
            mock_response = mock_get.return_value
            mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError(
                f"Not Found for url: {url}", response=mock_response
            )

            with self.assertRaises(StorageClientError) as cm:
                StorageClient.download_file_from_url(url)
            self.assertEqual(f"Not Found for url: {url}", str(cm.exception))

    def test_download_files(self):
        expected_result = [b"file1 contents", b"file2 contents"]
        self.client.client.get_object = MagicMock(
            side_effect=[
                MagicMock(read=MagicMock(return_value=expected_result[0])),
                MagicMock(read=MagicMock(return_value=expected_result[1])),
            ]
        )
        result = self.client.download_files(files=self.files, bucket=self.bucket)
        self.assertEqual(result, expected_result)

    def test_download_files_error(self):
        self.client.client.get_object = MagicMock(
            side_effect=S3Error(
                code="NoSuchKey",
                message="Key not found",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageFileNotFoundError):
            self.client.download_files(files=self.files, bucket=self.bucket)

    def test_download_files_anonymous_error(self):
        self.client.client.get_object = MagicMock(
            side_effect=S3Error(
                code="InvalidAccessKeyId",
                message="Access denied",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageClientError):
            self.client.download_files(files=self.files, bucket=self.bucket)

    def test_download_files_exception(self):
        self.client.client.get_object = MagicMock(
            side_effect=Exception("Connection error")
        )
        with self.assertRaises(StorageClientError):
            self.client.download_files(files=self.files, bucket=self.bucket)

    def test_upload_files(self):
        file3 = "file3 content"
        data_sha = hashlib.sha1(json.dumps("file3 content").encode("utf-8")).hexdigest()
        key3 = f"s3{data_sha}.json"

        self.client.client.stat_object = MagicMock(
            side_effect=S3Error(
                code="NoSuchKey",
                message="Object does not exist",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        self.client.client.put_object = MagicMock()
        result = self.client.upload_files(files=[file3], bucket=self.bucket)
        self.assertEqual(result, [key3])

    def test_upload_files_exist(self):
        file3 = "file3 content"
        data_sha = hashlib.sha1(json.dumps("file3 content").encode("utf-8")).hexdigest()
        key3 = f"s3{data_sha}.json"

        self.client.client.stat_object = MagicMock(
            side_effect=[{"_object_name": "1234567890"}]
        )
        self.client.client.put_object = MagicMock()
        result = self.client.upload_files(files=[file3], bucket=self.bucket)
        self.assertEqual(result, [key3])

    def test_upload_files_error(self):
        file3 = "file3 content"

        # HeadObject error
        self.client.client.head_object = MagicMock(
            side_effect=S3Error(
                code="InvalidAccessKeyId",
                message="Access denied",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageClientError):
            self.client.upload_files(files=[file3], bucket=self.bucket)

        # PutObject error
        self.client.client.upload_fileobj = MagicMock(
            side_effect=S3Error(
                code="InvalidAccessKeyId",
                message="Access denied",
                resource="",
                request_id="",
                host_id="",
                response="",
            )
        )
        with self.assertRaises(StorageClientError):
            self.client.upload_files(files=[file3], bucket=self.bucket)

    def test_bucket_exists(self):
        self.client.client.bucket_exists = MagicMock(side_effect=[True])
        result = self.client.bucket_exists(bucket=self.bucket)
        self.assertEqual(result, True)

    def test_bucket_exists_anonymous(self):
        client = StorageClient(
            endpoint_url=self.endpoint_url,
        )
        client.client.bucket_exists = MagicMock(side_effect=[True])
        result = client.bucket_exists(bucket=self.bucket)
        self.assertEqual(result, True)

    def test_bucket_not_exists(self):
        self.client.client.bucket_exists = MagicMock(side_effect=[False])
        result = self.client.bucket_exists(bucket=self.bucket)
        self.assertEqual(result, False)

    def test_bucket_error(self):
        self.client.client.bucket_exists = MagicMock(
            side_effect=Exception("Connection error")
        )
        with self.assertRaises(StorageClientError):
            self.client.bucket_exists(bucket=self.bucket)

    def test_list_objects(self):
        file1 = types.SimpleNamespace()
        file2 = types.SimpleNamespace()
        file1._object_name = "file1"
        file2._object_name = "file2"
        self.client.client.list_objects = MagicMock(side_effect=[[file1, file2]])
        result = self.client.list_objects(bucket=self.bucket)
        self.assertEqual(result, ["file1", "file2"])

    def test_list_objects_anonymous(self):
        client = StorageClient(
            endpoint_url=self.endpoint_url,
        )
        file1 = types.SimpleNamespace()
        file2 = types.SimpleNamespace()
        file1._object_name = "file1"
        file2._object_name = "file2"
        client.client.list_objects = MagicMock(side_effect=[[file1, file2]])
        result = client.list_objects(bucket=self.bucket)
        self.assertEqual(result, ["file1", "file2"])

    def test_list_objects_empty(self):
        self.client.client.list_objects = MagicMock(side_effect=[[]])
        result = self.client.list_objects(bucket=self.bucket)
        self.assertEqual(result, [])

    def test_list_objects_error(self):
        self.client.client.head_bucket = MagicMock(
            side_effect=Exception("Connection error")
        )
        with self.assertRaises(StorageClientError):
            self.client.list_objects(bucket=self.bucket)

    def test_list_objects_length(self):
        expected_length = random.randint(1, 10)
        mock_client = MagicMock()
        mock_client.list_objects.return_value = [
            types.SimpleNamespace(_object_name=f"file{i}")
            for i in range(expected_length)
        ]
        with patch("human_protocol_sdk.storage.Minio", return_value=mock_client):
            client = StorageClient(endpoint_url="https://example.com", credentials=None)
            object_list = client.list_objects(bucket="my-bucket")
            self.assertEqual(len(object_list), expected_length)

    def test_list_objects_length_error(self):
        expected_length = random.randint(1, 10)
        mock_client = MagicMock()
        mock_client.list_objects.return_value = [
            types.SimpleNamespace(_object_name=f"file{i}")
            for i in range(expected_length)
        ]
        with patch("human_protocol_sdk.storage.Minio", return_value=mock_client):
            client = StorageClient(endpoint_url="https://example.com", credentials=None)
            object_list = client.list_objects(bucket="my-bucket")
            if len(object_list) != expected_length:
                raise AssertionError(
                    f"Expected {expected_length} objects, but found {len(object_list)}"
                )
